# -*- coding: utf-8 -*-
# spusteni z cmd:
# C:\Python27\python.exe game.py <host:80> <max_speed>

# kdyz pri spousteni z cmd je zadan max_speed parametr, hra se spousti v automatic modu

# v pycharm jsem vytvoril 2 build configurace, "game" build configurace pousti hru v manualnim rezimu, "game-automatic" v automatickem, oboji na localhost
# Ovladani:
#           Motor nahoru: sipka nahoru
#           doleva:        sipka doleva
#           doprava:        sipka doprava





import threading

import pygame, time, random, traceback, urllib2, sys
import httplib
from pygame.locals import *

BLACK = (0, 0, 0)
WHITE = (255, 255, 255)
RED = (255, 0, 0)
GREEN = (0, 255, 0)
BLUE = (0, 0, 255)


def parse_server_response(response):
    # print response
    dic = {}
    response = response.split("\r\n")
    dic["id"] = response[0]
    xy = response[1].split()
    dic["x"] = float(xy[0])
    dic["y"] = float(xy[1])

    vx_vy = response[2].split()
    dic["vx"] = float(vx_vy[0])
    dic["vy"] = float(vx_vy[1])

    dic["p"] = float(response[3])

    l_r = response[4].split()
    dic["l"] = float(l_r[0])
    dic["r"] = float(l_r[1])

    return dic


def get(host, path):
    # return urllib2.urlopen(host + path).read().decode("utf-8")

    conn = httplib.HTTPConnection(host)
    conn.request("GET", path)

    res = conn.getresponse()
    text = res.read()
    res.close()
    conn.close()
    return  text


def wait_for_keypress():
    while True:
        for event in pygame.event.get():
            if event.type == QUIT:
                pygame.quit()
                sys.exit(0)

            elif event.type == KEYDOWN:
                return event.key
        pygame.display.update()
        fpsClock.tick(FPS)

    reset_state_variables()


def draw(game):
    scale = 1

    pygame.draw.rect(window, (20,20,20), (0, 0, 600, 600), 0)
    center = (game.r + game.l) / 2.
    disx = game.x - center
    disy = game.y

    scale1 = 1
    scale2 = 1


    if abs(disx) > 250:
        scale1 = 250/abs(disx)


    scale2 = 250 / abs(disy)
    scale = min(scale1, scale2)

    genx = 300 + int(disx * scale)
    geny = 600 - int(disy * scale)
    pygame.draw.rect(window, GREEN, (int(genx - scale*10), int(geny - scale*10), int(20*scale), int(20*scale)), 0)

    height = 3
    land_width = (game.r - game.l)*scale
    pygame.draw.rect(window, GREEN, (300 - land_width/2, 600 - height, land_width, height), 0)

    # print disx, scale
    # print scale


    #
    # pygame.draw.rect(window, GREEN, (int(game.x) - 10, WINDOWHEIGHT - (int(game.y) + 10), 20, 20), 0)
    # pygame.draw.circle(window, RED, (int(game.x), WINDOWHEIGHT - int(game.y)), 3)



    draw_stats(game)


def draw_stats(game):
    x = myfont.render("X:" + str(game.x), 1, WHITE)
    y = myfont.render("Y:" + str(game.y), 1, WHITE)
    vx = myfont.render("Vx:" + str(game.vx), 1, WHITE)
    vy = myfont.render("Vy:" + str(game.vy), 1, WHITE)
    p =  myfont.render("Palivo:" + str(game.p), 1, WHITE)

    window.blit(x, (600, 100))
    window.blit(y, (600, 150))
    window.blit(vx, (600, 200))
    window.blit(vy, (600, 250))
    window.blit(p, (600, 350))
    # window.blit(x, (600, 100))



def reset_state_variables():
    pygame.event.clear()
    global  KEY_LEFT, KEY_UP, KEY_RIGHT

    KEY_UP, KEY_LEFT, KEY_RIGHT = False, False, False





def lost_sequence():
    # render text
    label = myfont.render("You crashed your moonlander! Press spacebar to start a new game", 1, (255, 100, 0))
    window.blit(label, (100, 100))
    while True:
        if wait_for_keypress() == K_SPACE:
            break
    reset_state_variables()


def win_sequence():
    reset_state_variables()
    label = myfont.render("You Successfully landed!", 1, (0, 255, 0))
    window.blit(label, (100, 100))
    while True:
        if wait_for_keypress() == K_SPACE:
            break
    reset_state_variables()


# Pro lepsi plynulost hry se pozadavky na server pousti v paralelnim vlakne
# zatimco se provadi pozadavek na server, poloha, ktera je zobrazovana se pocita prubezne podle rovnic ze zadani
# pri vraceni response ze serveru se aktualizuji lokalne ulozene stavove promene (x, y, vx, vy...)
class RequestThread(threading.Thread):
    def __init__(self, game):
        threading.Thread.__init__(self)

    def run(self):
        # s = time.time()
        query = "/stav?id=" + game.id + "&nahoru=" + str(game.up) + "&doleva=" + str(game.left) + "&doprava=" + str(game.right)
        # print query
        response = get(game.url, query)
        # print(time.time() - s)
        # print("server response")

        if response.startswith("Krach"):
            game.status = "crash"
        elif response.startswith("Zdar"):
            game.status = "landed"
        else:
            params = parse_server_response(response)
            game.x, game.y = params["x"], params["y"]
            game.vx, game.vy = params["vx"], params["vy"]
            game.l, game.r = params["l"], params["r"]
            game.p = params["p"]

        game.request_running = False


class Game():
    global KEY_RIGHT, KEY_UP, KEY_LEFT
    request_running = False

    def __init__(self, adresa="127.0.0.1:8000"):
        self.url = adresa

    def start_request(self):
        self.request_running = True

    def start_game(self):
        reset_state_variables()

        params = parse_server_response(get(self.url, "/start"))
        self.id = params["id"]
        self.x, self.y = params["x"], params["y"]
        self.vx, self.vy = params["vx"], params["vy"]
        self.l, self.r = params["l"], params["r"]
        self.p = params["p"]
        self.left, self.right, self.up = 0., 0., 0.


        self.status = "running"
        self.last_time = time.time()


    def compute_new_coordinates(self):
        # print self.left, self.right, self.up

        elapsed = time.time() - self.last_time
        self.last_time = time.time()

        xnew = self.x + self.vx * elapsed + 1./2*(6*self.right - 6*self.left) * elapsed**2
        ynew = self.y + self.vy * elapsed + 1. / 2 * (6 * self.right - 6 * self.left) * elapsed ** 2

        vxnew = self.vx + (6*self.right * - 6*self.left) * elapsed
        vynew = self.vy + (-7 + 12 * self.up) * elapsed
        pnew = self.p - (1/2*self.left + 1/2*self.right + self.up) * elapsed

        self.x, self.y, self.vx, self.vy, self.p = xnew, ynew, vxnew, vynew, pnew


    def update(self):
        # returns "running" when game is still running, "crash" when crashed and "landed" when successfully lands

        if self.request_running == False:
            # start new request in paralel thread
            self.requestThread = RequestThread(game)
            self.request_running = True
            self.requestThread.start()

        self.compute_new_coordinates()






        return self.status



# for i in range(10):
#     s = time.time()
#     get("http://localhost:8000", "/start")
#     print(time.time() - s)

# set up pygame
pygame.init()

myfont = pygame.font.SysFont("monospace", 15)
smallFont = pygame.font.SysFont("monospace", 10)

# set up the window
WINDOWWIDTH = 800
WINDOWHEIGHT = 600
FPS = 40
fpsClock = pygame.time.Clock()
# set up the colors

KEY_LEFT = False
KEY_RIGHT = False
KEY_UP = False
# KEY_LEFT = False
# KEY_LEFT = False

AUTOMATIC = False
host = "127.0.0.1:8000"

if len(sys.argv) == 3:      # automatic mode, max speed was given
    host = sys.argv[1]
    max_speed = sys.argv[2]
    AUTOMATIC = True
elif len(sys.argv) == 2:        # custom host, not automatic mode
    host = sys.argv[1]







# game = Game()
game = Game(host)
game.start_game()

window = pygame.display.set_mode((WINDOWWIDTH, WINDOWHEIGHT), 0, 32)
try:
    while True:
        window.fill(BLACK)

        # print AUTOMATIC
        for event in pygame.event.get():
            if event.type == QUIT:
                pygame.quit()

            elif event.type == KEYDOWN:

                if event.key == K_UP:
                    KEY_UP = True

                if event.key == K_LEFT:
                    KEY_LEFT = True

                if event.key == K_RIGHT:
                    KEY_RIGHT = True

            elif event.type == KEYUP:

                if event.key == K_UP:
                    KEY_UP = False

                if event.key == K_LEFT:
                    KEY_LEFT = False

                if event.key == K_RIGHT:
                    KEY_RIGHT = False



        if AUTOMATIC:
            if (2*abs(game.y - 10)*5) ** 0.5 < abs(game.vy):
                KEY_UP = True




        # print KEY_UP, KEY_LEFT, KEY_RIGHT
        game.left, game.right, game.up = int(KEY_LEFT), int(KEY_RIGHT), int(KEY_UP)


        status = game.update()
        if status == "crash":
            lost_sequence()
            game.start_game()
            # print KEY_LEFT, KEY_RIGHT, KEY_UP


        elif status == "landed":
            win_sequence()
            game.start_game()
        elif status == "running":
            pass

        draw(game)
        pygame.display.update()
        fpsClock.tick(FPS)
except:
    traceback.print_exc()
    pygame.quit()
    raise

